#ifndef RHYME__H_
#define RHYME__H_

/**
  * @file RHYME.h
  *
  * Library to work with rhymes.
  *
  * @author Marcell Dietl, Sven Eric Panitz
  * @date 25.01.2010
  *
  **/

#include "WORDS.h"

struct RhymePair {
	/** Word that rhymes with word2. **/
	string word1;
	/** Word that rhymes with word1. **/
	string word2;
};

/** RP is used for pointers on RhymePair structures to increase readability. **/
typedef struct RhymePair* RP;

/**
  * A constructor function for RhymePair objects.
  *
  * The function stores two words, that rhyme with each other,
  * in a RhymePair structure.
  *
  * @param word1 The first word, that rhymes with word2.
  * @param word2 The second word, that rhymes with word1.
  *
  * @result A pointer to the newly created object.
  *
  **/
RP newRhymePair(string word1, string word2);

/**
  * A destructor function for RhymePair objects.
  *
  * The function releases the resources previously used to hold
  * a RhymePair object.
  *
  * @param this Pointer to the RhymePair object to be freed.
  *
  **/
void deleteRhymePair(RP this);

struct RhymePairList {
	/** Two words, that rhyme with each other, saved in a RhymePair structure. **/
	RP head;
	/** Pointer on the next element in the list. **/
	struct RhymePairList* tail;
};

/** RPList is used for pointers on RhymePairList structures to increase readability. **/
typedef struct RhymePairList* RPList;

/**
  * A constructor function for an empty RhymePairList object.
  *
  * The function creates a RhymePairList object, that does not contain
  * any value in the head and a tail, that points to NULL. Thus being the
  * last element in a list.
  *
  * @result A pointer to the newly created object.
  *
  **/
RPList nil();

/**
  * A constructor function for RhymePairList objects.
  *
  * The function creates a RhymePairList object with a pair of rhyming
  * words in the head - saved in a RhymePair object - and a tail, that
  * points to the next object.
  *
  * @param head A pointer to the RhymePair object.
  * @param tail A pointer to the next RhymePairList object.
  *
  * @result A pointer to the newly created object.
  *
  **/
RPList newRhymePairList(RP head, RPList tail);

/**
  * A destructor function for RhymePairList objects.
  *
  * The function recursively releases the resources previously
  * used to hold a RhymePairList object.
  *
  * @param this A pointer to the RhymePairList object.
  *
  **/
void deleteRhymePairList(RPList this);

/**
  * Create a list of rhymes from a WordList object.
  *
  * The function analyzes a WordList object for words, that
  * rhyme with each other and saves each pair in newly generated
  * RhymePairList object.
  *
  * @param ws A pointer to the WordList object.
  *
  * @result A pointer to the object, that contains all found rhymes.
  *
  **/
RPList fromWordList(WordList* ws);

/**
  * A function to find a spoonerism in a list of rhymes.
  *
  * The function analyzes a RhymePairList object to find a RhymePair,
  * that builds a spoonerism with a given RhymePair. Thus the second
  * RhymePair must start with the same consonants as the first one.
  *
  * @param this A pointer to the RhymePairList object.
  * @param rhymeWords A pointer to a RhymePair object.
  *
  * @result A pointer to a spoonerism of rhymeWords.
  *
  **/
RP spoonerism(RPList this, RP rhymeWords);

#endif
